###############################
## ABDpred Server
## Version 1.0
###############################




import warnings
import numpy as np
import csv
import copy
from sklearn.svm import SVC #SVM support vector import from sklearn
from sklearn.model_selection import StratifiedKFold #import for StratifiedKFold
from sklearn.metrics import recall_score
from sklearn.metrics import accuracy_score #import for accuracy score
from sklearn.metrics import confusion_matrix
from statistics import mean

warnings.filterwarnings('ignore')

def read_attribute(name) : #function defination of attribute of csv file
    f=open(name)
    reader=csv.DictReader(f)
    attr=reader.fieldnames
    f.close()
    return attr


def read_dataset(name, cols, sh): #function defination of attribute of csv file excluding header/attribute
    f=open(name)
    b=np.genfromtxt(f,delimiter=',',usecols=cols, skip_header=sh)
    f.close()
    return b

def replace_missing() : #replace missing value
    col_mean = np.nanmean(X, axis=0)
    x,y = np.where(np.isnan(X))
    for i in range(len(x)) :
        X[x[i], y[i]] = col_mean[y[i]]

def predict_with_threshold(estimator, X_test, threshold) : #threshold value calculation
    if threshold == None :
        return estimator.predict(X_test)
    des = estimator.decision_function(X_test)
    y_pred = []
    for val in des :
        if val<threshold :
            y_pred.append(0)
        else :
            y_pred.append(1)
    #print y_pred
    #print des
    return y_pred
            
name='train_1_1_work_1.csv'
a = read_attribute(name) # call of function of attribute of the csv file

X = read_dataset(name,range(1,18),1) #call of function for X
y= read_dataset(name, 0, 1) #call of function for y
replace_missing()

def write_csv(name,thesh, sen,acc,spec) :
    fileds=['Threshold', 'Sensitivity', 'Accuracy', 'Specificity']
    csvfile=open(name,'w+')
    writer=csv.DictWriter(csvfile, fieldnames=fileds)
    writer.writeheader()
    for i in range(len(sen)):
        row = {fileds[0]: thesh[i], fileds[1]: sen[i], fileds[2]: acc[i], fileds[3]: spec[i]}
        writer.writerow(row)
    csvfile.close()

learned_SVM_models = []
learned_NB_mdels=[]
learned_RF_models=[]

def learn_k_fold(k, model, learned_models) :
    test_indices = []
    skf = StratifiedKFold(n_splits=k)
    for train_index, test_index in skf.split(X,y):
        test_indices.append(copy.deepcopy(test_index))
        X_train = X[train_index]
        #X_test = X[test_index]
        y_train = y[train_index]
        #y_test = y[test_index]
        model.fit(X_train, y_train)
        learned_models.append(copy.deepcopy(model))
    return test_indices

def predict_k_fold(k, test_indices, learned_models, t=None):
    accuracies = []
    sensitivities=[]
    specificities=[]
    #for m in learned_models :
    #print len(learned_models), len(test_indices)
    for i in range(len(learned_models)):
        X_test = X[test_indices[i]]
        y_test = y[test_indices[i]]
        m = learned_models[i]
        y_pred = predict_with_threshold(m, X_test, t)
        accuracy = accuracy_score(y_test, y_pred)
        accuracies.append(accuracy)
        sensitivity = recall_score(y_test, y_pred)
        sensitivities.append(sensitivity)
        
        cf = confusion_matrix(y_test,y_pred)
        tn, fp, fn, tp = cf.ravel()
        specificity=(tn/1.0)/(fp+tn)
        specificities.append(specificity)
    #print t
    ##print accuracies
    print (t, mean(sensitivities), mean(accuracies),mean (specificities))
    return (mean(sensitivities), mean(accuracies),mean (specificities))

#model = SVC(class_weight='balanced', C=10, kernel='rbf', gamma=0.0001, max_iter=-1)

k=5

for c in [1,10, 250, 350, 360]:
    for g in [0.00001]:
            #print('C = ', c)
            #print('gamma = ', g)
            #c=1
            #g=0.0001
            model = SVC(class_weight='balanced', C=c, kernel='rbf', gamma=g, max_iter=-1)
            print ("when c is ", c)
            test_indices = learn_k_fold(k, model, learned_SVM_models)
    
            thresholds = [0 , 0.1, 0.3, 0.5, 0.7, 0.9, 2]
            accuracies = []
            sensitivities=[]
            specificities=[]
            print('Threshold','Sensitivity','Accuracy','Specificity')
            for t in thresholds:
                sens, specs, acc = predict_k_fold(k, test_indices, learned_SVM_models, t)
                accuracies.append(acc)
                sensitivities.append(sens)
                specificities.append(specs)
            learned_SVM_models = []
            diff = np.absolute(np.array(sensitivities) - np.array(specificities))
            max_index = diff.tolist().index(diff.min())
            print ('Max')
            print (thresholds[max_index], sensitivities[max_index], specificities[max_index], accuracies[max_index])
            write_csv('threshold.csv', thresholds, sensitivities, accuracies, specificities)

